@@ -31,10 +31,7 @@
 extern crate libc;
 
 #[cfg(windows)]
-extern crate winapi;
-
-#[cfg(target_os = "redox")]
-extern crate syscall;
+extern crate windows_sys;
 
 /// Returns a number that is unique to the calling thread.
 ///
@@ -46,23 +43,75 @@ pub fn get() -> usize {
     get_internal()
 }
 
-#[cfg(unix)]
+#[cfg(all(
+    unix,
+    not(any(
+        target_os = "macos",
+        target_os = "ios",
+        target_os = "tvos",
+        target_os = "watchos"
+    ))
+))]
 #[inline]
 fn get_internal() -> usize {
     unsafe { libc::pthread_self() as usize }
 }
 
+#[cfg(all(
+    unix,
+    any(
+        target_os = "macos",
+        target_os = "ios",
+        target_os = "tvos",
+        target_os = "watchos"
+    )
+))]
+#[inline]
+fn get_internal() -> usize {
+    let mut tid: u64 = 0;
+    unsafe {
+        libc::pthread_threadid_np(libc::pthread_self(), &mut tid as *mut u64);
+    };
+    tid as usize
+}
+
 #[cfg(windows)]
 #[inline]
 fn get_internal() -> usize {
-    unsafe { winapi::um::processthreadsapi::GetCurrentThreadId() as usize }
+    unsafe { windows_sys::Win32::System::Threading::GetCurrentThreadId() as usize }
 }
 
-#[cfg(target_os = "redox")]
+#[cfg(all(
+    target_arch = "wasm32",
+    target_vendor = "unknown",
+    target_os = "unknown",
+    not(target_feature = "atomics"),
+))]
 #[inline]
 fn get_internal() -> usize {
-    // Each thread has a separate pid on Redox.
-    syscall::getpid().unwrap()
+    0
+}
+
+#[cfg(any(
+    target_env = "sgx",
+    all(
+        target_arch = "wasm32",
+        target_vendor = "unknown",
+        target_os = "unknown",
+        target_feature = "atomics",
+    )
+))]
+#[inline]
+fn get_internal() -> usize {
+    use std::sync::atomic::{AtomicUsize, Ordering};
+
+    static COUNTER: AtomicUsize = AtomicUsize::new(0);
+
+    thread_local! {
+        static ID: usize = COUNTER.fetch_add(1, Ordering::Relaxed);
+    }
+
+    ID.with(|id| *id)
 }
 
 #[test]
@@ -71,9 +120,9 @@ fn distinct_threads_have_distinct_ids() {
     use std::thread;
 
     let (tx, rx) = mpsc::channel();
-    thread::spawn(move || tx.send(::get()).unwrap()).join().unwrap();
+    thread::spawn(move || tx.send(crate::get()).unwrap()).join().unwrap();
 
-    let main_tid = ::get();
+    let main_tid = crate::get();
     let other_tid = rx.recv().unwrap();
     assert!(main_tid != other_tid);
 }
